package org.apache.bcel.classfile;

import java.io.DataOutputStream;
import java.io.IOException;
import java.io.DataInput;

public final class ModuleExports implements Cloneable, Node
{
    private final int exportsIndex;
    private final int exportsFlags;
    private final int exportsToCount;
    private final int[] exportsToIndex;
    
    ModuleExports(final DataInput file) throws IOException {
        this.exportsIndex = file.readUnsignedShort();
        this.exportsFlags = file.readUnsignedShort();
        this.exportsToCount = file.readUnsignedShort();
        this.exportsToIndex = new int[this.exportsToCount];
        for (int i = 0; i < this.exportsToCount; ++i) {
            this.exportsToIndex[i] = file.readUnsignedShort();
        }
    }
    
    @Override
    public void accept(final Visitor v) {
        v.visitModuleExports(this);
    }
    
    public ModuleExports copy() {
        try {
            return (ModuleExports)this.clone();
        }
        catch (final CloneNotSupportedException ex) {
            return null;
        }
    }
    
    public void dump(final DataOutputStream file) throws IOException {
        file.writeShort(this.exportsIndex);
        file.writeShort(this.exportsFlags);
        file.writeShort(this.exportsToCount);
        for (final int entry : this.exportsToIndex) {
            file.writeShort(entry);
        }
    }
    
    @Override
    public String toString() {
        return "exports(" + this.exportsIndex + ", " + this.exportsFlags + ", " + this.exportsToCount + ", ...)";
    }
    
    public String toString(final ConstantPool constantPool) {
        final StringBuilder buf = new StringBuilder();
        final String packageName = constantPool.constantToString(this.exportsIndex, (byte)20);
        buf.append(Utility.compactClassName(packageName, false));
        buf.append(", ").append(String.format("%04x", this.exportsFlags));
        buf.append(", to(").append(this.exportsToCount).append("):\n");
        for (final int index : this.exportsToIndex) {
            final String moduleName = constantPool.getConstantString(index, (byte)19);
            buf.append("      ").append(Utility.compactClassName(moduleName, false)).append("\n");
        }
        return buf.substring();
    }
}
